package com.analogbit.puzzletools.client.slidingblocks.gui;

import com.analogbit.puzzletools.client.slidingblocks.state.Grid;
import com.analogbit.puzzletools.client.util.PreviousNextPanel.DISPLAY_MODE;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Button;

/**
 * Wizard panel that contains the grid builder. 
 */
public class InitialSetupPanel extends WizardPanel implements GridBuilderListener {

	private Button newButton = new Button("New Grid");
	private GridBuilderDisplay gridDisplay = new GridBuilderDisplay();

	public InitialSetupPanel() {
		super("Initial Setup", "Layout the initial state of the board. You can create blocks by dragging the left mouse button over empty space. Click on a block to get a popup menu. Blocks can be marked &quot;Locked In Place&quot; so the solver knows it may not move them.");

		add(gridDisplay);
		add(newButton);

		setPreviousDisplayMode(DISPLAY_MODE.HIDDEN);

		gridDisplay.addGridBuilderListener(this);
		
		newButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				Grid currentGrid = gridDisplay.getGrid();
				final NewGridDialogBox newDlg = new NewGridDialogBox(currentGrid.getWidth(), currentGrid.getHeight());
				newDlg.center();
				
				newDlg.addOkClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						gridDisplay.newGrid(newDlg.getWidthAsInt(), newDlg.getHeightAsInt());
						newDlg.hide();
					}
				});
				newDlg.addCancelClickHandler(new ClickHandler() {
					
					public void onClick(ClickEvent event) {
						newDlg.hide();;
					}
				});
				
				newDlg.show();
				
			}
		});
	}

	public Grid getGrid() {
		return gridDisplay.getGrid();
	}

	public void onGridBuildingChanged() {
		setNextDisplayMode(gridDisplay.getGrid().hasMovableBlocks() ? DISPLAY_MODE.NORMAL : DISPLAY_MODE.DISABLED);
	}

}
