package com.analogbit.puzzletools.client.slidingblocks.gui;

import java.util.List;

import com.analogbit.puzzletools.client.slidingblocks.state.Block;
import com.analogbit.puzzletools.client.slidingblocks.state.Grid;
import com.analogbit.puzzletools.client.slidingblocks.state.SolutionStep;
import com.google.gwt.user.client.Timer;

/**
 * Displays and animates the solution as a grid.
 */
public class SolutionGridDisplay extends GridDisplay {
	private Grid currentGrid;
	private List<SolutionStep> solution;
	private int currentIndex;
	private MoveTimer timer;
	
	public SolutionGridDisplay() {
	}

	public void setSolution(Grid originalGrid, List<SolutionStep> solution) {
		this.solution = solution;
		this.currentGrid = originalGrid.deepClone();
		this.currentIndex = 0;
		setGridDisplaying(currentGrid);
		timer = null;
	}
	
	@Override
	public boolean isDraggable(GridBlock block) {
		return false;
	}

	@Override
	public boolean isHighlightable(GridBlock block) {
		return false;
	}

	public void displayNext() {
		if (!hasNext()) {
			return;
		}
		
		SolutionStep step = solution.get(currentIndex);
		
		currentGrid.stepForward(step);
		currentIndex++;
		
		if (timer != null) {
			timer.finishIt();
		}
		
		Block blockMoved = currentGrid.getBlocks().get(step.getIndexMoved());
		GridBlock gridBlockMoved = getGridBlock(blockMoved);
		
		timer = new MoveTimer(gridBlockMoved, 
				blockMoved.getLeft()*GridBlock.CELL_SIZE,
				blockMoved.getTop()*GridBlock.CELL_SIZE);
		timer.scheduleRepeating(20);
		
	}

	public void displayPrevious() {
		if (!hasPrevious()) {
			return;
		}
		
		currentIndex--;
		SolutionStep step = solution.get(currentIndex);
		currentGrid.stepBackward(step);
		
		if (timer != null) {
			timer.finishIt();
		}
		
		Block blockMoved = currentGrid.getBlocks().get(step.getIndexMoved());
		GridBlock gridBlockMoved = getGridBlock(blockMoved);
		
		timer = new MoveTimer(gridBlockMoved, 
				blockMoved.getLeft()*GridBlock.CELL_SIZE,
				blockMoved.getTop()*GridBlock.CELL_SIZE);
		timer.scheduleRepeating(20);
	}

	
	public boolean hasNext() {
		return currentIndex < solution.size();
	}
	
	public boolean hasPrevious() {
		return currentIndex > 0;
	}
	
	/**
	 * Timer for animating a block sliding.
	 */
	class MoveTimer extends Timer {
		public static final int NUM_STEPS = 6;
		int step;
		GridBlock block;
		int targetX;
		int targetY;
		
		public MoveTimer(GridBlock block, int targetX, int targetY) {
			this.block = block;
			this.targetX = targetX;
			this.targetY = targetY;
			
			step = 0;
		}

		@Override
		public void run() {
			step++;

			if (step == NUM_STEPS) {
				finishIt();
			} else {
				int blockLeft = getWidgetLeft(block);
				int blockTop = getWidgetTop(block);
				
				int posX = (blockLeft*3 + targetX) / 4;
				int posY = (blockTop*3 + targetY) / 4;

				setWidgetPosition(block, posX, posY);
			}
		}
		
		public void finishIt() {
			step = NUM_STEPS;
			setWidgetPosition(block, targetX, targetY);
			this.cancel();
		}
	}

}
