package com.analogbit.puzzletools.client.slidingblocks.gui;

import java.util.ArrayList;

import com.analogbit.puzzletools.client.util.PreviousNextPanel;
import com.analogbit.puzzletools.client.util.PreviousNextPanel.DISPLAY_MODE;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * A panel treated like a setup wizard. It has a content panel and a previous, next option. 
 */
public class WizardPanel extends VerticalPanel implements SourcesPhaseEvents {
	
	private ArrayList<PhaseListener> listeners = new ArrayList<PhaseListener>();
	private PreviousNextPanel prevNextPanel = new PreviousNextPanel(
			PreviousNextPanel.DISPLAY_MODE.NORMAL,
			PreviousNextPanel.DISPLAY_MODE.NORMAL);
	
	private VerticalPanel wizardArea = new VerticalPanel();
	private HTML description = new HTML();
	
	public WizardPanel(String name, String htmlDescription) {
		this.setHorizontalAlignment(ALIGN_CENTER);
		wizardArea.setHorizontalAlignment(ALIGN_CENTER);
		addStyleName("wizardPanel");
		Label nameLabel = new Label(name);
		nameLabel.addStyleName("title");
		description.setHTML(htmlDescription);
		description.addStyleName("description");
		wizardArea.addStyleName("wizardArea");
		super.add(nameLabel);
		super.add(description);
		super.add(prevNextPanel);
		super.add(wizardArea);
		
		prevNextPanel.addNextClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				fireNextStage();
			}
		});

		prevNextPanel.addPreviousClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				firePreviousStage();
			}
		});

	}
	
	protected void setDescription(String htmlDescription) {
		description.setHTML(htmlDescription);
	}
	
	private void fireNextStage() {
		for (int i = 0; i < listeners.size(); i++) {
			listeners.get(i).onPhaseNext();
		}
	}
	
	private void firePreviousStage() {
		for (int i = 0; i < listeners.size(); i++) {
			listeners.get(i).onPhasePrev();
		}
	}

	public void addPhaseListener(PhaseListener listener) {
		listeners.add(listener);
	}
	
	public boolean removePhaseListener(PhaseListener listener) {
		return listeners.remove(listener);
	}
	
	public void setNextDisplayMode(DISPLAY_MODE nextDisplayMode) {
		prevNextPanel.setNextDisplayMode(nextDisplayMode);
	}

	public void setPreviousDisplayMode(DISPLAY_MODE previousDisplayMode) {
		prevNextPanel.setPreviousDisplayMode(previousDisplayMode);
	}

	@Override
	public void add(Widget w) {
		wizardArea.add(w);
	}

	@Override
	public boolean remove(Widget w) {
		return wizardArea.remove(w);
	}

	@Override
	public void clear() {
		wizardArea.clear();
	}

}
