package com.analogbit.puzzletools.client.slidingblocks.state;

import java.util.ArrayList;

import com.analogbit.puzzletools.client.util.Color;

public class GridWithListeners extends Grid {
	private ArrayList<GridListener> listeners = new ArrayList<GridListener>();
	
	public GridWithListeners(int width, int height, ArrayList<Block> blocks,
			short depth, Grid previous) {
		super(width, height, blocks, depth, previous);
	}

	public GridWithListeners(int width, int height, ArrayList<Block> blocks) {
		super(width, height, blocks);
	}

	public GridWithListeners(int width, int height) {
		super(width, height);
	}

	public void addGridListener(GridListener listener) {
		listeners.add(listener);
	}
	
	public boolean removeGridListener(GridListener listener) {
		return listeners.remove(listener);
	}

	@Override
	public boolean moveBlockIfLegal(Block block, int x, int y) {
		int oldX = block.getX();
		int oldY = block.getY();
		boolean moved = super.moveBlockIfLegal(block, x, y);
		
		if (moved) {
			for (GridListener listener : listeners) {
				listener.onBlockMoved(block, oldX, oldY);
			}
		}
		return moved;
	}

	@Override
	public void addBlock(Block block) {
		super.addBlock(block);

		for (GridListener listener : listeners) {
			listener.onBlockAdded(block);
		}
	}

	@Override
	public void setBlockColor(Block block, Color color) {
		super.setBlockColor(block, color);
		fireBlockChanged(block);
	}

	@Override
	public void setBlockImmobile(Block block, boolean immobile) {
		super.setBlockImmobile(block, immobile);
		fireBlockChanged(block);
	}
	
	@Override
	public boolean removeBlock(Block block) {
		boolean removed = super.removeBlock(block);
		if (removed) {
			for (GridListener listener : listeners) {
				listener.onBlockRemoved(block);
			}
		}
		return removed;
	}

	//super.replaceBlockIfLegal does remove and add, so we get this one for free
	/*
	@Override
	public boolean replaceBlockIfLegal(Block oldBlock, Block newBlock) {
		boolean replaced = super.replaceBlockIfLegal(oldBlock, newBlock);
		if (replaced) {
			for (GridListener listener : listeners) {
				listener.onBlockRemoved(oldBlock);
			}
			for (GridListener listener : listeners) {
				listener.onBlockAdded(newBlock);
			}			
		}	
		return replaced;
	}
	
	//same here
	@Override
	public boolean addBlockIfLegal(Block block) {
		boolean added = super.addBlockIfLegal(block);
		if (added) {
			for (GridListener listener : listeners) {
				listener.onBlockAdded(block);
			}
		}
		return added;
	}
	
	*/

	private void fireBlockChanged(Block block) {
		for (GridListener listener : listeners) {
			listener.onBlockChanged(block);
		}
	}
}
